@@ -815,6 +815,7 @@ struct uwsgi_logger *uwsgi_get_logger_from_id(char *id) {
 void uwsgi_logit_lf(struct wsgi_request *wsgi_req) {
 	struct uwsgi_logchunk *logchunk = uwsgi.logchunks;
 	ssize_t rlen = 0;
+	const char *empty_var = "-";
 	while (logchunk) {
 		int pos = logchunk->vec;
 		// raw string
@@ -851,6 +852,11 @@ void uwsgi_logit_lf(struct wsgi_request *wsgi_req) {
 				uwsgi.logvectors[wsgi_req->async_id][pos].iov_len = 0;
 			}
 		}
+
+		if (uwsgi.logvectors[wsgi_req->async_id][pos].iov_len == 0 && logchunk->type != 0) {
+			uwsgi.logvectors[wsgi_req->async_id][pos].iov_base = (char *) empty_var;
+			uwsgi.logvectors[wsgi_req->async_id][pos].iov_len = 1; 
+		}
 		logchunk = logchunk->next;
 	}
 
@@ -862,7 +868,9 @@ void uwsgi_logit_lf(struct wsgi_request *wsgi_req) {
 	while (logchunk) {
 		if (logchunk->free) {
 			if (uwsgi.logvectors[wsgi_req->async_id][logchunk->vec].iov_len > 0) {
-				free(uwsgi.logvectors[wsgi_req->async_id][logchunk->vec].iov_base);
+				if (uwsgi.logvectors[wsgi_req->async_id][logchunk->vec].iov_base != empty_var) {
+					free(uwsgi.logvectors[wsgi_req->async_id][logchunk->vec].iov_base);
+				}
 			}
 		}
 		logchunk = logchunk->next;
@@ -923,17 +931,38 @@ void uwsgi_build_log_format(char *format) {
 
 }
 
-ssize_t uwsgi_lf_status(struct wsgi_request *wsgi_req, char **buf) {
+static ssize_t uwsgi_lf_status(struct wsgi_request *wsgi_req, char **buf) {
 	*buf = uwsgi_num2str(wsgi_req->status);
 	return strlen(*buf);
 }
 
-ssize_t uwsgi_lf_epoch(struct wsgi_request * wsgi_req, char **buf) {
+static ssize_t uwsgi_lf_rsize(struct wsgi_request *wsgi_req, char **buf) {
+        *buf = uwsgi_num2str(wsgi_req->response_size);
+        return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_hsize(struct wsgi_request *wsgi_req, char **buf) {
+        *buf = uwsgi_num2str(wsgi_req->headers_size);
+        return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_size(struct wsgi_request *wsgi_req, char **buf) {
+        *buf = uwsgi_num2str(wsgi_req->headers_size+wsgi_req->response_size);
+        return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_cl(struct wsgi_request *wsgi_req, char **buf) {
+        *buf = uwsgi_num2str(wsgi_req->post_cl);
+        return strlen(*buf);
+}
+
+
+static ssize_t uwsgi_lf_epoch(struct wsgi_request * wsgi_req, char **buf) {
 	*buf = uwsgi_num2str(uwsgi_now());
 	return strlen(*buf);
 }
 
-ssize_t uwsgi_lf_ctime(struct wsgi_request * wsgi_req, char **buf) {
+static ssize_t uwsgi_lf_ctime(struct wsgi_request * wsgi_req, char **buf) {
 	*buf = uwsgi_malloc(26);
 #ifdef __sun__
 	ctime_r((const time_t *) &wsgi_req->start_of_request_in_sec, *buf, 26);
@@ -943,23 +972,98 @@ ssize_t uwsgi_lf_ctime(struct wsgi_request * wsgi_req, char **buf) {
 	return 24;
 }
 
-ssize_t uwsgi_lf_time(struct wsgi_request * wsgi_req, char **buf) {
+static ssize_t uwsgi_lf_time(struct wsgi_request * wsgi_req, char **buf) {
 	*buf = uwsgi_num2str(wsgi_req->start_of_request / 1000000);
 	return strlen(*buf);
 }
 
+static ssize_t uwsgi_lf_ltime(struct wsgi_request * wsgi_req, char **buf) {
+        *buf = uwsgi_malloc(64);
+        time_t now = wsgi_req->start_of_request / 1000000;
+        size_t ret = strftime(*buf, 64, "%d/%b/%Y:%H:%M:%S %z", localtime(&now));
+        if (ret == 0) {
+                *buf[0] = 0;
+                return 0;
+        }
+        return ret;
+}
 
 
-ssize_t uwsgi_lf_micros(struct wsgi_request * wsgi_req, char **buf) {
+
+static ssize_t uwsgi_lf_micros(struct wsgi_request * wsgi_req, char **buf) {
 	*buf = uwsgi_num2str(wsgi_req->end_of_request - wsgi_req->start_of_request);
 	return strlen(*buf);
 }
 
-ssize_t uwsgi_lf_msecs(struct wsgi_request * wsgi_req, char **buf) {
+static ssize_t uwsgi_lf_msecs(struct wsgi_request * wsgi_req, char **buf) {
 	*buf = uwsgi_num2str((wsgi_req->end_of_request - wsgi_req->start_of_request) / 1000);
 	return strlen(*buf);
 }
 
+static ssize_t uwsgi_lf_pid(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(uwsgi.mypid);
+	return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_wid(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(uwsgi.mywid);
+	return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_switches(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(wsgi_req->switches);
+	return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_vars(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(wsgi_req->var_cnt);
+	return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_core(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(wsgi_req->async_id);
+	return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_vsz(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(uwsgi.workers[uwsgi.mywid].vsz_size);
+	return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_rss(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(uwsgi.workers[uwsgi.mywid].rss_size);
+	return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_vszM(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(uwsgi.workers[uwsgi.mywid].vsz_size / 1024 / 1024);
+	return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_rssM(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(uwsgi.workers[uwsgi.mywid].rss_size / 1024 / 1024);
+	return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_pktsize(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(wsgi_req->uh.pktsize);
+	return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_modifier1(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(wsgi_req->uh.modifier1);
+	return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_modifier2(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(wsgi_req->uh.modifier2);
+	return strlen(*buf);
+}
+
+static ssize_t uwsgi_lf_headers(struct wsgi_request * wsgi_req, char **buf) {
+	*buf = uwsgi_num2str(wsgi_req->header_cnt);
+	return strlen(*buf);
+}
 
 void uwsgi_add_logchunk(int variable, int pos, char *ptr, size_t len) {
 
@@ -1018,11 +1122,39 @@ void uwsgi_add_logchunk(int variable, int pos, char *ptr, size_t len) {
 			logchunk->pos = offsetof(struct wsgi_request, protocol);
 			logchunk->pos_len = offsetof(struct wsgi_request, protocol_len);
 		}
+		else if (!uwsgi_strncmp(ptr, len, "uagent", 6)) {
+                        logchunk->pos = offsetof(struct wsgi_request, user_agent);
+                        logchunk->pos_len = offsetof(struct wsgi_request, user_agent_len);
+                }
+                else if (!uwsgi_strncmp(ptr, len, "referer", 7)) {
+                        logchunk->pos = offsetof(struct wsgi_request, referer);
+                        logchunk->pos_len = offsetof(struct wsgi_request, referer_len);
+                }
 		else if (!uwsgi_strncmp(ptr, len, "status", 6)) {
 			logchunk->type = 3;
 			logchunk->func = uwsgi_lf_status;
 			logchunk->free = 1;
 		}
+		else if (!uwsgi_strncmp(ptr, len, "rsize", 5)) {
+                        logchunk->type = 3;
+                        logchunk->func = uwsgi_lf_rsize;
+                        logchunk->free = 1;
+                }
+                else if (!uwsgi_strncmp(ptr, len, "hsize", 5)) {
+                        logchunk->type = 3;
+                        logchunk->func = uwsgi_lf_hsize;
+                        logchunk->free = 1;
+                }
+                else if (!uwsgi_strncmp(ptr, len, "size", 4)) {
+                        logchunk->type = 3;
+                        logchunk->func = uwsgi_lf_size;
+                        logchunk->free = 1;
+                }
+                else if (!uwsgi_strncmp(ptr, len, "cl", 2)) {
+                        logchunk->type = 3;
+                        logchunk->func = uwsgi_lf_cl;
+                        logchunk->free = 1;
+                }
 		else if (!uwsgi_strncmp(ptr, len, "micros", 6)) {
 			logchunk->type = 3;
 			logchunk->func = uwsgi_lf_micros;
@@ -1038,6 +1170,11 @@ void uwsgi_add_logchunk(int variable, int pos, char *ptr, size_t len) {
 			logchunk->func = uwsgi_lf_time;
 			logchunk->free = 1;
 		}
+		else if (!uwsgi_strncmp(ptr, len, "ltime", 5)) {
+                        logchunk->type = 3;
+                        logchunk->func = uwsgi_lf_ltime;
+                        logchunk->free = 1;
+                }
 		else if (!uwsgi_strncmp(ptr, len, "ctime", 5)) {
 			logchunk->type = 3;
 			logchunk->func = uwsgi_lf_ctime;
@@ -1048,6 +1185,71 @@ void uwsgi_add_logchunk(int variable, int pos, char *ptr, size_t len) {
 			logchunk->func = uwsgi_lf_epoch;
 			logchunk->free = 1;
 		}
+		else if (!uwsgi_strncmp(ptr, len, "pid", 3)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_pid;
+			logchunk->free = 1;
+		}
+		else if (!uwsgi_strncmp(ptr, len, "wid", 3)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_wid;
+			logchunk->free = 1;
+		}
+		else if (!uwsgi_strncmp(ptr, len, "switches", 8)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_switches;
+			logchunk->free = 1;
+		}
+		else if (!uwsgi_strncmp(ptr, len, "vars", 4)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_vars;
+			logchunk->free = 1;
+		}
+		else if (!uwsgi_strncmp(ptr, len, "core", 4)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_core;
+			logchunk->free = 1;
+		}
+		else if (!uwsgi_strncmp(ptr, len, "vsz", 3)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_vsz;
+			logchunk->free = 1;
+		}
+		else if (!uwsgi_strncmp(ptr, len, "rss", 3)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_rss;
+			logchunk->free = 1;
+		}
+		else if (!uwsgi_strncmp(ptr, len, "vszM", 4)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_vszM;
+			logchunk->free = 1;
+		}
+		else if (!uwsgi_strncmp(ptr, len, "rssM", 4)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_rssM;
+			logchunk->free = 1;
+		}
+		else if (!uwsgi_strncmp(ptr, len, "pktsize", 7)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_pktsize;
+			logchunk->free = 1;
+		}
+		else if (!uwsgi_strncmp(ptr, len, "modifier1", 9)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_modifier1;
+			logchunk->free = 1;
+		}
+		else if (!uwsgi_strncmp(ptr, len, "modifier2", 9)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_modifier2;
+			logchunk->free = 1;
+		}
+		else if (!uwsgi_strncmp(ptr, len, "headers", 7)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_headers;
+			logchunk->free = 1;
+		}
 		// logvar
 		else {
 			logchunk->type = 2;
