@@ -630,7 +630,7 @@ int bind_to_tcp(char *socket_name, int listen_queue, char *tcp_port) {
 	}
 
 #ifdef __linux__
-	long somaxconn = uwsgi_num_from_file("/proc/sys/net/core/somaxconn");
+	long somaxconn = uwsgi_num_from_file("/proc/sys/net/core/somaxconn", 1);
 	if (somaxconn > 0 && uwsgi.listen_queue > somaxconn) {
 		uwsgi_log("Listen queue size is greater than the system max net.core.somaxconn (%i).\n", somaxconn);
 		uwsgi_nuclear_blast();
@@ -1611,7 +1611,7 @@ void uwsgi_map_sockets() {
 				exit(1);
 			}
 			if (fd != uwsgi_sock->fd) {
-				if (dup2(fd, uwsgi_sock->fd)) {
+				if (dup2(fd, uwsgi_sock->fd) < 0) {
 					uwsgi_error("dup2()");
 					exit(1);
 				}
@@ -1702,14 +1702,14 @@ void uwsgi_bind_sockets() {
 		gsa.sa = (struct sockaddr *) &usa;
 		if (!uwsgi.skip_zero && !getsockname(0, gsa.sa, &socket_type_len)) {
 			if (gsa.sa->sa_family == AF_UNIX) {
-				uwsgi_sock = uwsgi_new_socket(usa.sa_un.sun_path);
+				uwsgi_sock = uwsgi_new_socket(uwsgi_getsockname(0));
 				uwsgi_sock->family = AF_UNIX;
 				uwsgi_sock->fd = 0;
 				uwsgi_sock->bound = 1;
 				uwsgi_log("uwsgi socket %d inherited UNIX address %s fd 0\n", uwsgi_get_socket_num(uwsgi_sock), uwsgi_sock->name);
 			}
 			else {
-				uwsgi_sock = uwsgi_new_socket(uwsgi_concat2("::", ""));
+				uwsgi_sock = uwsgi_new_socket(uwsgi_getsockname(0));
 				uwsgi_sock->family = AF_INET;
 				uwsgi_sock->fd = 0;
 				uwsgi_sock->bound = 1;
@@ -1723,7 +1723,7 @@ void uwsgi_bind_sockets() {
 				exit(1);
 			}
 			if (fd != 0) {
-				if (dup2(fd, 0)) {
+				if (dup2(fd, 0) < 0) {
 					uwsgi_error("dup2()");
 					exit(1);
 				}
