@@ -230,7 +230,7 @@ ssize_t uwsgi_gevent_hook_input_read(struct wsgi_request *wsgi_req, char *tmp_bu
         	}
         	Py_DECREF(ret);
 
-        	ret = PyObject_CallMethod(timer, "start", "OO", current, timer);
+        	ret = PyObject_CallMethod(timer, "again", "OO", current, timer);
         	if (!ret) {
                 	stop_the_watchers_and_clear
                 	return -1;
@@ -253,6 +253,8 @@ ssize_t uwsgi_gevent_hook_input_read(struct wsgi_request *wsgi_req, char *tmp_bu
 		UWSGI_RELEASE_GIL;	
                 ssize_t rlen = read(wsgi_req->poll.fd, tmp_buf+*tmp_pos, remains);
                 if (rlen <= 0) {
+			if (rlen < 0)
+				uwsgi_error("[uwsgi-gevent] read()");
                         UWSGI_GET_GIL
 			stop_the_watchers_and_clear
                         return -1;
@@ -263,6 +265,11 @@ ssize_t uwsgi_gevent_hook_input_read(struct wsgi_request *wsgi_req, char *tmp_bu
 		stop_the_watchers
         }
 
+	Py_DECREF(current);
+	Py_DECREF(current_greenlet);
+	Py_DECREF(watcher);
+	Py_DECREF(timer);
+
         return *tmp_pos;
 
 }
@@ -291,7 +298,7 @@ ssize_t uwsgi_gevent_hook_input_readline(struct wsgi_request *wsgi_req, char *re
         }
         Py_DECREF(ret);
 
-        ret = PyObject_CallMethod(timer, "start", "OO", current, timer);
+        ret = PyObject_CallMethod(timer, "again", "OO", current, timer);
         if (!ret) {
         	stop_the_watchers_and_clear
 		return -1;
@@ -328,6 +335,10 @@ void uwsgi_gevent_nb_write(struct wsgi_request *wsgi_req, PyObject *str) {
 	PyObject *ret;
 	char *content = PyString_AsString(str);
 	size_t content_len = PyString_Size(str);
+
+	// do not try to write empty chunks
+	if (content_len == 0) return;
+
 	/// create a watcher for writes
 	PyObject *watcher = PyObject_CallMethod(ugevent.hub_loop, "io", "ii", wsgi_req->poll.fd, 2);
 	if (!watcher) goto error;
@@ -353,7 +364,7 @@ void uwsgi_gevent_nb_write(struct wsgi_request *wsgi_req, PyObject *str) {
 		}
 		Py_DECREF(ret);
 
-		ret = PyObject_CallMethod(timer, "start", "OO", current, timer);
+		ret = PyObject_CallMethod(timer, "again", "OO", current, timer);
 		if (!ret) {
 			stop_the_watchers_and_clear
 			goto error;
@@ -420,7 +431,7 @@ PyObject *uwsgi_gevent_wait(PyObject *watcher, PyObject *timer, PyObject *curren
 	Py_DECREF(ret);
 
 	// start the timeout handler
-	ret = PyObject_CallMethod(timer, "start", "OO", current, timer);
+	ret = PyObject_CallMethod(timer, "again", "OO", current, timer);
 	if (!ret) return NULL;
 	Py_DECREF(ret);
 
