@@ -24,6 +24,7 @@ struct uwsgi_http {
 #ifdef UWSGI_SSL
 	int https_export_cert;
 #endif
+	struct uwsgi_string_list *stud_prefix;
 
 } uhttp;
 
@@ -134,6 +135,7 @@ struct uwsgi_option http_options[] = {
 	{"http-stats-server", required_argument, 0, "run the http router stats server", uwsgi_opt_set_str, &uhttp.cr.stats_server, 0},
 	{"http-ss", required_argument, 0, "run the http router stats server", uwsgi_opt_set_str, &uhttp.cr.stats_server, 0},
 	{"http-harakiri", required_argument, 0, "enable http router harakiri", uwsgi_opt_set_int, &uhttp.cr.harakiri, 0},
+	{"http-stud-prefix", required_argument, 0, "expect a stud prefix (1byte family + 4/16 bytes address) on connections from the specified address", uwsgi_opt_add_addr_list, &uhttp.stud_prefix, 0},
 	{0, 0, 0, 0, 0, 0, 0},
 };
 
@@ -175,6 +177,11 @@ struct http_session {
         size_t post_buf_len;
         off_t post_buf_pos;
 
+	// 1 (family) + 4/16 (addr)
+	char stud_prefix[17];
+	size_t stud_prefix_remains;
+	size_t stud_prefix_pos;
+
 
 };
 
@@ -354,6 +361,11 @@ int http_parse(struct http_session *h_session, size_t http_req_len) {
 	// UWSGI_ROUTER
 	if (http_add_uwsgi_var(h_session, "UWSGI_ROUTER", 12, "http", 4)) return -1;
 
+	// stud HTTPS
+	if (h_session->stud_prefix_pos > 0) {
+		if (http_add_uwsgi_var(h_session, "HTTPS", 5, "on", 2)) return -1;
+	}
+
 #ifdef UWSGI_SSL
 	// HTTPS (adapted from nginx)
 	if (h_session->cs.ugs->mode == UWSGI_HTTP_SSL) {
@@ -514,14 +526,14 @@ ssize_t hr_read_ssl_body(struct corerouter_session * cs) {
         struct http_session *hs = (struct http_session *) cs;
         int ret = SSL_read(hs->ssl, hs->post_buf->buf, hs->post_buf_max);
         if (ret > 0) {
-                // fix waiting
-                if (cs->event_hook_write) {
-                        uwsgi_cr_hook_write(cs, NULL);
-                }
 		int ret2 = SSL_pending(hs->ssl);
                 if (ret2 > 0) {
-			if (uwsgi_buffer_fix(hs->post_buf, hs->post_buf->len + ret2 )) return -1;
+			if (uwsgi_buffer_fix(hs->post_buf, hs->post_buf->len + ret2 )) {
+				uwsgi_log("[uwsgi-https] cannot fix the buffer to %d\n", hs->post_buf->len + ret2);
+				return -1;
+			}
                         if (SSL_read(hs->ssl, hs->post_buf->buf + ret, ret2) != ret2) {
+				uwsgi_log("[uwsgi-https] SSL_read() on %d bytes of pending data failed\n", ret2);
                                 return -1;
                         }
                         ret += ret2;
@@ -539,8 +551,8 @@ ssize_t hr_read_ssl_body(struct corerouter_session * cs) {
         if (err == SSL_ERROR_WANT_READ) {
                 if (cs->event_hook_write) {
                         uwsgi_cr_hook_write(cs, NULL);
-                        uwsgi_cr_hook_read(cs, hr_read_ssl_body);
                 }
+                uwsgi_cr_hook_read(cs, hr_read_ssl_body);
                 errno = EINPROGRESS;
                 return -1;
         }
@@ -548,8 +560,8 @@ ssize_t hr_read_ssl_body(struct corerouter_session * cs) {
         else if (err == SSL_ERROR_WANT_WRITE) {
                 if (cs->event_hook_read) {
                         uwsgi_cr_hook_read(cs, NULL);
-                        uwsgi_cr_hook_write(cs, hr_read_ssl_body);
                 }
+                uwsgi_cr_hook_write(cs, hr_read_ssl_body);
                 errno = EINPROGRESS;
                 return -1;
         }
@@ -599,9 +611,6 @@ ssize_t hr_write_ssl_response(struct corerouter_session * cs) {
 
 	if (ret > 0) {
         	cs->buffer_pos += ret;
-		if (cs->event_hook_read) {
-			uwsgi_cr_hook_read(cs, NULL);
-		}
 		// could be a partial write
 		uwsgi_cr_hook_write(cs, hr_write_ssl_response);
         	// ok this response chunk is sent, let's wait for another one
@@ -616,16 +625,16 @@ ssize_t hr_write_ssl_response(struct corerouter_session * cs) {
         if (err == SSL_ERROR_WANT_READ) {
 		if (cs->event_hook_write) {
 			uwsgi_cr_hook_write(cs, NULL);
-			uwsgi_cr_hook_read(cs, hr_write_ssl_response);
 		}
+		uwsgi_cr_hook_read(cs, hr_write_ssl_response);
                 errno = EINPROGRESS;
                 return -1;
         }
         else if (err == SSL_ERROR_WANT_WRITE) {
 		if (cs->event_hook_read) {
 			uwsgi_cr_hook_read(cs, NULL);
-			uwsgi_cr_hook_write(cs, hr_write_ssl_response);
 		}
+		uwsgi_cr_hook_write(cs, hr_write_ssl_response);
                 errno = EINPROGRESS;
                 return -1;
         }
@@ -822,9 +831,6 @@ ssize_t hr_send_expect_continue(struct corerouter_session * cs) {
 		if (ret > 0) {
 			len = ret;
 			cs->buffer_pos += ret;
-			if (cs->event_hook_read) {
-                        	uwsgi_cr_hook_read(cs, NULL);
-                	}
                 	// could be a partial write
                 	uwsgi_cr_hook_write(cs, hr_send_expect_continue);
 			goto done;
@@ -833,16 +839,16 @@ ssize_t hr_send_expect_continue(struct corerouter_session * cs) {
         	if (err == SSL_ERROR_WANT_READ) {
                 	if (cs->event_hook_write) {
                         	uwsgi_cr_hook_write(cs, NULL);
-                        	uwsgi_cr_hook_read(cs, hr_write_ssl_response);
                 	}
+                        uwsgi_cr_hook_read(cs, hr_write_ssl_response);
                 	errno = EINPROGRESS;
                 	return -1;
         	}
         	else if (err == SSL_ERROR_WANT_WRITE) {
                 	if (cs->event_hook_read) {
                         	uwsgi_cr_hook_read(cs, NULL);
-                        	uwsgi_cr_hook_write(cs, hr_write_ssl_response);
                 	}
+                        uwsgi_cr_hook_write(cs, hr_write_ssl_response);
                 	errno = EINPROGRESS;
                 	return -1;
         	}
@@ -916,19 +922,21 @@ ssize_t hr_recv_http_ssl(struct corerouter_session * cs) {
 	// be sure buffer does not grow over 64k
         cs->buffer->limit = UMAX16;
         // try to always leave 4k available
-        if (uwsgi_buffer_ensure(cs->buffer, uwsgi.page_size)) return -1;
+        if (uwsgi_buffer_ensure(cs->buffer, uwsgi.page_size))  {
+		uwsgi_log("[uwsgi-https] cannot ensure the buffer (size: %d)\n", cs->buffer->len);	 
+		return -1;
+	}
 	struct http_session *hs = (struct http_session *) cs;
 	int ret = SSL_read(hs->ssl, cs->buffer->buf + cs->buffer_pos, cs->buffer->len - cs->buffer_pos);
 	if (ret > 0) {
-		// fix waiting
-		if (cs->event_hook_write) {
-			uwsgi_cr_hook_write(cs, NULL);
-			uwsgi_cr_hook_read(cs, hr_recv_http_ssl);
-		}
 		int ret2 = SSL_pending(hs->ssl);
 		if (ret2 > 0) {
-			if (uwsgi_buffer_fix(cs->buffer, cs->buffer->len + ret2 )) return -1;
+			if (uwsgi_buffer_fix(cs->buffer, cs->buffer->len + ret2 )) {
+				uwsgi_log("[uwsgi-https] cannot fix the buffer to %d\n", cs->buffer->len + ret2 );
+				return -1;
+			}
 			if (SSL_read(hs->ssl, cs->buffer->buf + cs->buffer_pos + ret, ret2) != ret2) {
+				uwsgi_log("[uwsgi-https] SSL_read() on %d bytes of pending data failed\n", ret2);
 				return -1;
 			}
 			ret += ret2;
@@ -941,8 +949,8 @@ ssize_t hr_recv_http_ssl(struct corerouter_session * cs) {
 	if (err == SSL_ERROR_WANT_READ) {
                 if (cs->event_hook_write) {
                         uwsgi_cr_hook_write(cs, NULL);
-                        uwsgi_cr_hook_read(cs, hr_recv_http_ssl);
                 }
+                uwsgi_cr_hook_read(cs, hr_recv_http_ssl);
                 errno = EINPROGRESS;
                 return -1;
         }
@@ -950,8 +958,8 @@ ssize_t hr_recv_http_ssl(struct corerouter_session * cs) {
         else if (err == SSL_ERROR_WANT_WRITE) {
                 if (cs->event_hook_read) {
                         uwsgi_cr_hook_read(cs, NULL);
-                        uwsgi_cr_hook_write(cs, hr_recv_http_ssl);
                 }
+                uwsgi_cr_hook_write(cs, hr_recv_http_ssl);
                 errno = EINPROGRESS;
                 return -1;
         }
@@ -1071,6 +1079,31 @@ void hr_session_close(struct corerouter_session *cs) {
 	}
 }
 
+static ssize_t hr_recv_stud4(struct corerouter_session * cs) {
+	struct http_session *hs = (struct http_session *) cs;
+	ssize_t len = read(cs->fd, hs->stud_prefix + hs->stud_prefix_pos, hs->stud_prefix_remains - hs->stud_prefix_pos);
+	if (len < 0) {
+                cr_try_again;
+                uwsgi_error("hr_recv_stud4()");
+                return -1;
+        }
+
+	hs->stud_prefix_pos += len;
+
+        if (hs->stud_prefix_pos == hs->stud_prefix_remains) {
+		if (hs->stud_prefix[0] != AF_INET) {
+			uwsgi_log("[uwsgi-http] invalid stud prefix\n");
+			return -1;
+		}
+		// set the passed ip address
+		memcpy(&hs->ip_addr, hs->stud_prefix + 1, 4);
+		uwsgi_cr_hook_read(cs, hr_recv_http);
+        }
+
+        return len;
+
+}
+
 #ifdef UWSGI_SSL
 void hr_session_ssl_close(struct corerouter_session *cs) {
 	hr_session_close(cs);
@@ -1092,6 +1125,8 @@ void hr_session_ssl_close(struct corerouter_session *cs) {
                 X509_free(hs->ssl_client_cert);
         }
 
+	// clear the errors (otherwise they could be propagated)
+	ERR_clear_error();
         SSL_free(hs->ssl);
 }
 #endif
@@ -1101,6 +1136,17 @@ void http_alloc_session(struct uwsgi_corerouter *ucr, struct uwsgi_gateway_socke
 	cs->modifier1 = uhttp.modifier1;
 	if (sa && sa->sa_family == AF_INET) {
 		hs->ip_addr = ((struct sockaddr_in *) sa)->sin_addr.s_addr;
+
+		struct uwsgi_string_list *usl = uhttp.stud_prefix;
+		while(usl) {
+			if (!memcmp(&hs->ip_addr, usl->value, 4)) {
+				hs->stud_prefix_remains = 5;
+				uwsgi_cr_hook_read(cs, hr_recv_stud4);
+				break;
+			}
+			usl = usl->next;
+		}
+
 	}
 
 	hs->rnrn = 0;
@@ -1122,7 +1168,9 @@ void http_alloc_session(struct uwsgi_corerouter *ucr, struct uwsgi_gateway_socke
 	}
 	else {
 #endif
-		uwsgi_cr_hook_read(cs, hr_recv_http);
+		if (!cs->event_hook_read) {
+			uwsgi_cr_hook_read(cs, hr_recv_http);
+		}
 		cs->close = hr_session_close;
 #ifdef UWSGI_SSL
 	}
