@@ -21,7 +21,12 @@ XS(XS_error) {
 
         psgi_check_args(0);
 
-        ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->error)[wsgi_req->async_id]);
+	if (uwsgi.threads > 1) {
+        	ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->error)[wsgi_req->async_id]);
+	}
+	else {
+        	ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->error)[0]);
+	}
         XSRETURN(1);
 }
 
@@ -32,7 +37,12 @@ XS(XS_input) {
 	struct uwsgi_app *wi = &uwsgi_apps[wsgi_req->app_id];
         psgi_check_args(0);
 
-        ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->input)[wsgi_req->async_id]);
+	if (uwsgi.threads > 1) {
+        	ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->input)[wsgi_req->async_id]);
+	}
+	else {
+        	ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->input)[0]);
+	}
         XSRETURN(1);
 }
 
@@ -66,7 +76,12 @@ XS(XS_stream)
 		while (psgi_response(wsgi_req, response) != UWSGI_OK);
 
 		SvREFCNT_dec(response);
-                ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->stream)[wsgi_req->async_id]);
+		if (uwsgi.threads > 1) {
+                	ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->stream)[wsgi_req->async_id]);
+		}
+		else {
+                	ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->stream)[0]);
+		}
                 XSRETURN(1);
 	}
 	else {
@@ -413,6 +428,23 @@ int init_psgi_app(struct wsgi_request *wsgi_req, char *app, uint16_t app_len, Pe
 			}
 		}
 
+		if (uperl.argv_items || uperl.argv_item) {
+			AV *uperl_argv = GvAV(PL_argvgv);
+			if (uperl.argv_items) {
+				char *argv_list = uwsgi_str(uperl.argv_items);
+				char *p = strtok(argv_list, " ");
+				while(p) {
+					av_push(uperl_argv, newSVpv(p, 0));
+					p = strtok(NULL, " ");
+				}
+			}
+			struct uwsgi_string_list *usl = uperl.argv_item;
+			while(usl) {
+				av_push(uperl_argv, newSVpv(usl->value, usl->len));
+				usl = usl->next;
+			}
+		}
+
 		SV *dollar_zero = get_sv("0", GV_ADD);
 		sv_setsv(dollar_zero, newSVpv(app, app_len));
 
