@@ -16,6 +16,11 @@ struct uwsgi_option uwsgi_perl_options[] = {
         {"psgi", required_argument, 0, "load a psgi app", uwsgi_opt_set_str, &uperl.psgi, 0},
         {"perl-no-die-catch", no_argument, 0, "do not catch $SIG{__DIE__}", uwsgi_opt_true, &uperl.no_die_catch, 0},
         {"perl-local-lib", required_argument, 0, "set perl locallib path", uwsgi_opt_set_str, &uperl.locallib, 0},
+#ifdef PERL_VERSION_STRING
+        {"perl-version", no_argument, 0, "print perl version", uwsgi_opt_print, PERL_VERSION_STRING, UWSGI_OPT_IMMEDIATE},
+#endif
+	{"perl-args", required_argument, 0, "add items (space separated) to @ARGV", uwsgi_opt_set_str, &uperl.argv_items, 0},
+	{"perl-arg", required_argument, 0, "add an item to @ARGV", uwsgi_opt_add_string_list, &uperl.argv_item, 0},
         {0, 0, 0, 0, 0, 0, 0},
 
 };
@@ -57,7 +62,12 @@ SV *uwsgi_perl_call_stream(SV *func) {
         ENTER;
         SAVETMPS;
         PUSHMARK(SP);
-        XPUSHs( sv_2mortal(newRV((SV*) ((SV **)wi->responder0)[wsgi_req->async_id])));
+	if (uwsgi.threads > 1) {
+        	XPUSHs( sv_2mortal(newRV((SV*) ((SV **)wi->responder0)[wsgi_req->async_id])));
+	}
+	else {
+        	XPUSHs( sv_2mortal(newRV((SV*) ((SV **)wi->responder0)[0])));
+	}
         PUTBACK;
 
 	call_sv( func, G_SCALAR | G_EVAL);
@@ -291,7 +301,12 @@ SV *build_psgi_env(struct wsgi_request *wsgi_req) {
 	
 	if (!hv_store(env, "psgix.input.buffered", 20, newSViv(wsgi_req->body_as_file), 0)) goto clear;
 
-	if (!hv_store(env, "psgix.logger", 12,newRV((SV*) ((SV **)wi->responder1)[wsgi_req->async_id]) ,0)) goto clear;
+	if (uwsgi.threads > 1) {
+		if (!hv_store(env, "psgix.logger", 12,newRV((SV*) ((SV **)wi->responder1)[wsgi_req->async_id]) ,0)) goto clear;
+	}
+	else {
+		if (!hv_store(env, "psgix.logger", 12,newRV((SV*) ((SV **)wi->responder1)[0]) ,0)) goto clear;
+	}
 
 	if (uwsgi.master_process) {
 		if (!hv_store(env, "psgix.harakiri", 14, newSViv(1), 0)) goto clear;
@@ -426,8 +441,15 @@ int uwsgi_perl_request(struct wsgi_request *wsgi_req) {
 	struct uwsgi_app *wi = &uwsgi_apps[wsgi_req->app_id];
 	wi->requests++;
 
-	if (((PerlInterpreter **)wi->interpreter)[wsgi_req->async_id] != uperl.main[wsgi_req->async_id]) {
-		PERL_SET_CONTEXT(((PerlInterpreter **)wi->interpreter)[wsgi_req->async_id]);
+	if (uwsgi.threads < 2) {
+		if (((PerlInterpreter **)wi->interpreter)[0] != uperl.main[0]) {
+			PERL_SET_CONTEXT(((PerlInterpreter **)wi->interpreter)[0]);
+		}
+	}
+	else {
+		if (((PerlInterpreter **)wi->interpreter)[wsgi_req->async_id] != uperl.main[wsgi_req->async_id]) {
+			PERL_SET_CONTEXT(((PerlInterpreter **)wi->interpreter)[wsgi_req->async_id]);
+		}
 	}
 
 	ENTER;
@@ -436,7 +458,12 @@ int uwsgi_perl_request(struct wsgi_request *wsgi_req) {
 	wsgi_req->async_environ = build_psgi_env(wsgi_req);
 	if (!wsgi_req->async_environ) goto clear;
 
-	wsgi_req->async_result = psgi_call(wsgi_req, ((SV **)wi->callable)[wsgi_req->async_id], wsgi_req->async_environ);
+	if (uwsgi.threads > 1) {
+		wsgi_req->async_result = psgi_call(wsgi_req, ((SV **)wi->callable)[wsgi_req->async_id], wsgi_req->async_environ);
+	}
+	else {
+		wsgi_req->async_result = psgi_call(wsgi_req, ((SV **)wi->callable)[0], wsgi_req->async_environ);
+	}
 	if (!wsgi_req->async_result) goto clear;
 
 	if (SvTYPE((AV *)wsgi_req->async_result) == SVt_PVCV) {
@@ -469,8 +496,15 @@ clear:
 	LEAVE;
 
 	// restore main interpreter if needed
-	if (((PerlInterpreter **)wi->interpreter)[wsgi_req->async_id] != uperl.main[wsgi_req->async_id]) {
-		PERL_SET_CONTEXT(uperl.main[wsgi_req->async_id]);
+	if (uwsgi.threads > 1) {
+		if (((PerlInterpreter **)wi->interpreter)[wsgi_req->async_id] != uperl.main[wsgi_req->async_id]) {
+			PERL_SET_CONTEXT(uperl.main[wsgi_req->async_id]);
+		}
+	}
+	else {
+		if (((PerlInterpreter **)wi->interpreter)[0] != uperl.main[0]) {
+			PERL_SET_CONTEXT(uperl.main[0]);
+		}
 	}
 
 	return UWSGI_OK;
@@ -554,6 +588,10 @@ void uwsgi_perl_post_fork() {
 		sv_setiv(GvSV(tmpgv), (IV)getpid());
 		SvREADONLY_on(GvSV(tmpgv));
 	}
+
+	if (uperl.postfork) {
+		uwsgi_perl_run_hook(uperl.postfork);
+	}
 }
 
 int uwsgi_perl_mount_app(char *mountpoint, char *app) {
@@ -621,6 +659,51 @@ int uwsgi_perl_signal_handler(uint8_t sig, void *handler) {
 	return ret;
 }
 
+void uwsgi_perl_run_hook(SV *hook) {
+	dSP;
+        ENTER;
+        SAVETMPS;
+        PUSHMARK(SP);
+        XPUSHs( hook );
+        PUTBACK;
+
+        call_sv( SvRV(hook), G_DISCARD);
+
+        if(SvTRUE(ERRSV)) {
+                uwsgi_log("[uwsgi-perl error] %s\n", SvPV_nolen(ERRSV));
+		return;
+        }
+
+        SPAGAIN;
+        PUTBACK;
+        FREETMPS;
+        LEAVE;
+}
+
+static void uwsgi_perl_atexit() {
+	if (uwsgi.mywid == -1) goto realstuff;
+
+        // if hijacked do not run atexit hooks
+        if (uwsgi.workers[uwsgi.mywid].hijacked)
+                return;
+
+        // if busy do not run atexit hooks
+        if (uwsgi.workers[uwsgi.mywid].busy)
+                return;
+
+#ifdef UWSGI_ASYNC
+        // managing atexit in async mode is a real pain...skip it for now
+        if (uwsgi.async > 1)
+                return;
+#endif
+
+realstuff:
+
+	if (uperl.atexit) {
+		uwsgi_perl_run_hook(uperl.atexit);
+	}
+}
+
 struct uwsgi_plugin psgi_plugin = {
 
 	.name = "psgi",
@@ -641,5 +724,7 @@ struct uwsgi_plugin psgi_plugin = {
 	.after_request = uwsgi_perl_after_request,
 	.enable_threads = uwsgi_perl_enable_threads,
 
+	.atexit = uwsgi_perl_atexit,
+
 	.magic = uwsgi_perl_magic,
 };
