@@ -2,6 +2,7 @@
 
 extern struct uwsgi_server uwsgi;
 extern struct uwsgi_plugin psgi_plugin;
+extern struct uwsgi_perl uperl;
 
 #ifdef UWSGI_ASYNC
 
@@ -171,6 +172,28 @@ XS(XS_register_signal) {
 	
 }
 
+XS(XS_postfork) {
+        dXSARGS;
+
+        psgi_check_args(1);
+
+	uperl.postfork = newRV_inc(ST(0));
+
+        XSRETURN_YES;
+}
+
+XS(XS_atexit) {
+        dXSARGS;
+
+        psgi_check_args(1);
+
+        uperl.atexit = newRV_inc(ST(0));
+
+        XSRETURN_YES;
+}
+
+
+
 XS(XS_log) {
 
 	dXSARGS;
@@ -267,6 +290,41 @@ XS(XS_signal_wait) {
 	XSRETURN_YES;
 }
 
+XS(XS_add_timer) {
+
+        dXSARGS;
+
+        psgi_check_args(2);
+
+        uint8_t uwsgi_signal = SvIV(ST(0));
+        int seconds = SvIV(ST(1));
+
+        if (uwsgi_add_timer(uwsgi_signal, seconds)) {
+                croak("unable to register timer");
+                XSRETURN_UNDEF;
+        }
+
+        XSRETURN(1);
+}
+
+XS(XS_add_rb_timer) {
+
+        dXSARGS;
+
+        psgi_check_args(2);
+
+        uint8_t uwsgi_signal = SvIV(ST(0));
+        int seconds = SvIV(ST(1));
+
+        if (uwsgi_signal_add_rb_timer(uwsgi_signal, seconds, 0)) {
+                croak("unable to register rb timer");
+                XSRETURN_UNDEF;
+        }
+
+        XSRETURN(1);
+}
+
+
 
 void init_perl_embedded_module() {
 	psgi_xs(reload);
@@ -282,5 +340,9 @@ void init_perl_embedded_module() {
 	psgi_xs(signal);
 	psgi_xs(register_signal);
 	psgi_xs(signal_wait);
+	psgi_xs(postfork);
+	psgi_xs(atexit);
+	psgi_xs(add_timer);
+	psgi_xs(add_rb_timer);
 }
 
