@@ -160,6 +160,7 @@ struct uwsgi_python {
 	void (*gil_get) (void);
 	void (*gil_release) (void);
 	int auto_reload;
+	char *tracebacker;
 	struct uwsgi_string_list *auto_reload_ignore;
 #endif
 
@@ -185,6 +186,11 @@ struct uwsgi_python {
 	char *pyrun;
 	int start_response_nodelay;
 
+	void (*hook_write_string)(struct wsgi_request *, PyObject *);
+	ssize_t (*hook_wsgi_input_read)(struct wsgi_request *, char *, size_t, size_t *);
+	ssize_t (*hook_wsgi_input_readline)(struct wsgi_request *, char *, size_t);
+
+	char *programname;
 };
 
 
@@ -209,9 +215,7 @@ int manage_python_response(struct wsgi_request *);
 int uwsgi_python_call(struct wsgi_request *, PyObject *, PyObject *);
 PyObject *python_call(PyObject *, PyObject *, int, struct wsgi_request *);
 
-#ifdef UWSGI_SENDFILE
 PyObject *py_uwsgi_sendfile(PyObject *, PyObject *);
-#endif
 
 PyObject *py_uwsgi_write(PyObject *, PyObject *);
 PyObject *py_uwsgi_spit(PyObject *, PyObject *);
@@ -270,9 +274,15 @@ void uwsgi_python_reset_random_seed(void);
 
 char *uwsgi_pythonize(char *);
 void *uwsgi_python_autoreloader_thread(void *);
+void *uwsgi_python_tracebacker_thread(void *);
 
 int uwsgi_python_manage_exceptions(void);
 int uwsgi_python_do_send_headers(struct wsgi_request *);
+void *uwsgi_python_tracebacker_thread(void *);
+PyObject *uwsgi_python_setup_thread(char *);
+
+ssize_t uwsgi_python_hook_simple_input_read(struct wsgi_request *, char *, size_t, size_t *);
+ssize_t uwsgi_python_hook_simple_input_readline(struct wsgi_request *, char *, size_t);
 
 #ifdef UWSGI_PYPY
 #undef UWSGI_MINTERPRETERS
