@@ -34,8 +34,23 @@ extern "C" {
 
 #define wsgi_req_time ((wsgi_req->end_of_request-wsgi_req->start_of_request)/1000)
 
-#define thunder_lock if (uwsgi.threads > 1 && !uwsgi.is_et) {pthread_mutex_lock(&uwsgi.thunder_mutex);}
-#define thunder_unlock if (uwsgi.threads > 1 && !uwsgi.is_et) {pthread_mutex_unlock(&uwsgi.thunder_mutex);}
+#define thunder_lock if (!uwsgi.is_et) {\
+                        if (uwsgi.use_thunder_lock) {\
+                                uwsgi_lock(uwsgi.the_thunder_lock);\
+                        }\
+                        else if (uwsgi.threads > 1) {\
+                                pthread_mutex_lock(&uwsgi.thunder_mutex);\
+                        }\
+                    }
+
+#define thunder_unlock if (!uwsgi.is_et) {\
+                        if (uwsgi.use_thunder_lock) {\
+                                uwsgi_unlock(uwsgi.the_thunder_lock);\
+                        }\
+                        else if (uwsgi.threads > 1) {\
+                                pthread_mutex_unlock(&uwsgi.thunder_mutex);\
+                        }\
+                        }
 
 #define uwsgi_check_scheme(file) (!uwsgi_startswith(file, "emperor://", 10) || !uwsgi_startswith(file, "http://", 7) || !uwsgi_startswith(file, "data://", 7) || !uwsgi_startswith(file, "sym://", 6) || !uwsgi_startswith(file, "fd://", 5) || !uwsgi_startswith(file, "exec://", 7) || !uwsgi_startswith(file, "section://", 10))
 
@@ -295,6 +310,8 @@ extern int pivot_root(const char *new_root, const char *put_old);
 #include <openssl/err.h>
 #endif
 
+#include <glob.h>
+
 
 struct uwsgi_buffer {
 	char *buf;
@@ -309,6 +326,7 @@ struct uwsgi_string_list {
 	size_t len;
 	uint64_t custom;
 	uint64_t custom2;
+	void *custom_ptr;
 	struct uwsgi_string_list *next;
 };
 
@@ -394,6 +412,25 @@ struct uwsgi_regexp_list {
 };
 #endif
 
+struct uwsgi_rbtree {
+        struct uwsgi_rb_timer *root;
+        struct uwsgi_rb_timer *sentinel;
+};
+
+struct uwsgi_rb_timer {
+        uint8_t color;
+        struct uwsgi_rb_timer *parent;
+        struct uwsgi_rb_timer *left;
+        struct uwsgi_rb_timer *right;
+        uint64_t value;
+        void *data;
+};
+
+struct uwsgi_rbtree *uwsgi_init_rb_timer(void);
+struct uwsgi_rb_timer *uwsgi_min_rb_timer(struct uwsgi_rbtree *, struct uwsgi_rb_timer *);
+struct uwsgi_rb_timer *uwsgi_add_rb_timer(struct uwsgi_rbtree *, uint64_t, void *);
+void uwsgi_del_rb_timer(struct uwsgi_rbtree *, struct uwsgi_rb_timer *);
+
 
 union uwsgi_sockaddr {
 	struct sockaddr sa;
@@ -850,6 +887,9 @@ struct uwsgi_route {
 	void *data2;
 	size_t data2_len;
 
+	void *data3;
+	size_t data3_len;
+
 	// 64bit value for custom usage
 	uint64_t custom;
 
@@ -905,6 +945,7 @@ struct uwsgi_alarm_ll {
 struct uwsgi_alarm_log {
 	pcre *pattern;
         pcre_extra *pattern_extra;
+	int negate;
 	struct uwsgi_alarm_ll *alarms;
 	struct uwsgi_alarm_log *next;
 };
@@ -991,6 +1032,12 @@ struct wsgi_request {
 	char *authorization;
 	uint16_t authorization_len;
 
+	char *user_agent;
+	uint16_t user_agent_len;
+
+	char *referer;
+	uint16_t referer_len;
+
 	char *script;
 	uint16_t script_len;
 	char *module;
@@ -1262,6 +1309,9 @@ struct uwsgi_server {
 
 	int reuse_port;
 
+	uint64_t http_modifier1;
+	uint64_t http_modifier2;
+
 	// enable lazy mode
 	int lazy;
 	// enable lazy-apps mode
@@ -1289,6 +1339,7 @@ struct uwsgi_server {
 
 	// true if run under the emperor
 	int has_emperor;
+	char *emperor_procname;
 	int emperor_fd;
 	int emperor_queue;
 	int emperor_tyrant;
@@ -1308,6 +1359,7 @@ struct uwsgi_server {
 	pid_t emperor_pid;
 	int	emperor_broodlord;
 	int	emperor_broodlord_count;
+	uint64_t emperor_broodlord_num;
 	char *emperor_stats;
 	int 	emperor_stats_fd;
 	struct uwsgi_string_list *vassals_templates;
@@ -1546,7 +1598,7 @@ struct uwsgi_server {
 	struct uwsgi_async_request *async_runqueue_last;
 	int async_runqueue_cnt;
 
-	struct rb_root *rb_async_timeouts;
+	struct uwsgi_rbtree *rb_async_timeouts;
 
 	int async_queue_unused_ptr;
 	struct wsgi_request **async_queue_unused;
@@ -1670,9 +1722,12 @@ struct uwsgi_server {
 
 #ifdef UWSGI_THREADING
 	// avoid thundering herd in threaded modes
+	pthread_mutex_t thunder_mutex;
 	pthread_mutex_t six_feet_under_lock;
 	pthread_mutex_t lock_static;
 #endif
+	int use_thunder_lock;
+	struct uwsgi_lock_item *the_thunder_lock;
 
 
 	/* the list of workers */
@@ -1856,8 +1911,6 @@ struct uwsgi_server {
 	size_t queue_filesize;
 	int queue_store_sync;
 
-	pthread_mutex_t thunder_mutex;
-
 	int locks;
 
 	struct uwsgi_lock_item *cache_lock;
@@ -2587,74 +2640,6 @@ struct uwsgi_subscribe_req {
 	uint16_t base_len;
 };
 
-#ifndef _NO_UWSGI_RB
-
-/*
-
-  *** uWSGI, stripped down version of Linux rbtree ***
-
-
-  Red Black Trees
-  (C) 1999  Andrea Arcangeli <andrea@suse.de>
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-
-*/
-
-struct rb_node
-{
-        unsigned long  rb_parent_color;
-#define RB_RED          0
-#define RB_BLACK        1
-        struct rb_node *rb_right;
-        struct rb_node *rb_left;
-} __attribute__((aligned(sizeof(long))));
-    /* The alignment might seem pointless, but allegedly CRIS needs it */
-
-struct rb_root
-{
-        struct rb_node *rb_node;
-};
-
-#define rb_parent(r)   ((struct rb_node *)((r)->rb_parent_color & ~3))
-#define rb_color(r)   ((r)->rb_parent_color & 1)
-#define rb_is_red(r)   (!rb_color(r))
-#define rb_is_black(r) rb_color(r)
-#define rb_set_red(r)  do { (r)->rb_parent_color &= ~1; } while (0)
-#define rb_set_black(r)  do { (r)->rb_parent_color |= 1; } while (0)
-
-void rb_insert_color(struct rb_node *, struct rb_root *);
-void rb_erase(struct rb_node *, struct rb_root *);
-
-void rb_link_node(struct rb_node *, struct rb_node *,
-                                struct rb_node **);
-
-struct uwsgi_rb_timer {
-
-	struct rb_node rbt;
-
-	time_t key;
-	void *data;
-};
-
-struct rb_root *uwsgi_init_rb_timer(void);
-struct uwsgi_rb_timer *uwsgi_add_rb_timer(struct rb_root *, time_t, void *);
-struct uwsgi_rb_timer *uwsgi_min_rb_timer(struct rb_root *);
-
-#endif
-
 void uwsgi_nuclear_blast();
 
 void uwsgi_unix_signal(int, void (*)(int));
@@ -2808,7 +2793,7 @@ int uwsgi_get_shared_socket_num(struct uwsgi_socket *);
 
 #ifdef __linux__
 void uwsgi_set_cgroup(void);
-long uwsgi_num_from_file(char *);
+long uwsgi_num_from_file(char *, int);
 #endif
 
 void uwsgi_add_sockets_to_queue(int, int);
@@ -2993,6 +2978,8 @@ void uwsgi_opt_set_str(char *, char *, void *);
 void uwsgi_opt_set_logger(char *, char *, void *);
 void uwsgi_opt_set_str_spaced(char *, char *, void *);
 void uwsgi_opt_add_string_list(char *, char *, void *);
+void uwsgi_opt_add_string_list_custom(char *, char *, void *);
+void uwsgi_opt_add_addr_list(char *, char *, void *);
 void uwsgi_opt_add_dyn_dict(char *, char *, void *);
 #ifdef UWSGI_PCRE
 void uwsgi_opt_pcre_jit(char *, char *, void *);
@@ -3073,6 +3060,7 @@ void uwsgi_opt_noop(char *, char *, void *);
 
 void uwsgi_opt_logic(char *, char *, void *);
 int uwsgi_logic_opt_for(char *, char *);
+int uwsgi_logic_opt_for_glob(char *, char *);
 int uwsgi_logic_opt_if_env(char *, char *);
 int uwsgi_logic_opt_if_not_env(char *, char *);
 int uwsgi_logic_opt_if_opt(char *, char *);
@@ -3168,7 +3156,7 @@ struct uwsgi_stats_pusher_instance;
 
 struct uwsgi_stats_pusher {
 	char *name;
-	void (*func)(struct uwsgi_stats_pusher_instance *, char *, size_t);
+	void (*func)(struct uwsgi_stats_pusher_instance *, time_t, char *, size_t);
 	struct uwsgi_stats_pusher *next;
 };
 
@@ -3176,6 +3164,7 @@ struct uwsgi_stats_pusher_instance {
 	struct uwsgi_stats_pusher *pusher;
 	char *arg;
 	void *data;
+	int raw;
 	int configured;
 	int freq;
 	time_t last_run;
@@ -3184,13 +3173,14 @@ struct uwsgi_stats_pusher_instance {
 
 struct uwsgi_thread;
 void uwsgi_stats_pusher_loop(struct uwsgi_thread *);
-void uwsgi_stats_pusher_file(struct uwsgi_stats_pusher_instance *, char *, size_t);
-void uwsgi_stats_pusher_socket(struct uwsgi_stats_pusher_instance *, char *, size_t);
+void uwsgi_stats_pusher_file(struct uwsgi_stats_pusher_instance *, time_t, char *, size_t);
+void uwsgi_stats_pusher_socket(struct uwsgi_stats_pusher_instance *, time_t, char *, size_t);
 
 void uwsgi_stats_pusher_setup(void);
 void uwsgi_send_stats(int, struct uwsgi_stats * (*func)(void));
 struct uwsgi_stats *uwsgi_master_generate_stats(void);
-void uwsgi_register_stats_pusher(char *, void(*) (struct uwsgi_stats_pusher_instance *, char *, size_t));
+struct uwsgi_stats_pusher *uwsgi_register_stats_pusher(char *, void(*) (struct uwsgi_stats_pusher_instance *, time_t, char *, size_t));
+struct uwsgi_stats_pusher_instance *uwsgi_stats_pusher_add(struct uwsgi_stats_pusher *, char *);
 
 struct uwsgi_stats *uwsgi_stats_new(size_t);
 int uwsgi_stats_symbol(struct uwsgi_stats *, char);
@@ -3216,6 +3206,7 @@ void manage_cluster_message(char *, int);
 void uwsgi_opt_add_custom_option(char *, char *, void *);
 void uwsgi_opt_cflags(char *, char *, void *);
 void uwsgi_opt_connect_and_read(char *, char *, void *);
+void uwsgi_opt_extract(char *, char *, void *);
 
 struct uwsgi_string_list *uwsgi_string_list_has_item(struct uwsgi_string_list *, char *, size_t);
 
@@ -3363,9 +3354,10 @@ int uwsgi_buffer_append(struct uwsgi_buffer *, char *, size_t);
 int uwsgi_buffer_fix(struct uwsgi_buffer *, size_t);
 int uwsgi_buffer_ensure(struct uwsgi_buffer *, size_t);
 void uwsgi_buffer_destroy(struct uwsgi_buffer *);
+int uwsgi_buffer_num64(struct uwsgi_buffer *, int64_t);
 
 void uwsgi_httpize_var(char *, size_t);
-struct uwsgi_buffer *uwsgi_to_http(struct wsgi_request *, char *, uint16_t);
+struct uwsgi_buffer *uwsgi_to_http(struct wsgi_request *, char *, uint16_t, char *, uint16_t);
 
 ssize_t uwsgi_pipe(int, int, int);
 ssize_t uwsgi_pipe_sized(int, int, size_t, int);
@@ -3482,6 +3474,8 @@ void simple_loop_run_int(int);
 int uwsgi_valid_fd(int);
 void uwsgi_close_all_fds(void);
 
+int uwsgi_write_nb(int, char *, size_t, int);
+
 void uwsgi_check_emperor(void);
 #ifdef UWSGI_AS_SHARED_LIBRARY
 int uwsgi_init(int, char **, char **);
