@@ -1,6 +1,6 @@
 # uWSGI build system
 
-uwsgi_version = '1.4.3'
+uwsgi_version = '1.4.9'
 
 import os
 import re
@@ -345,12 +345,16 @@ def build_uwsgi(uc, print_only=False):
                     pass
 
                 for cfile in up.GCC_LIST:
-                    if not cfile.endswith('.a'):
+                    if cfile.endswith('.a'):
+                        gcc_list.append(cfile)
+                    elif not cfile.endswith('.c') and not cfile.endswith('.cc') and not cfile.endswith('.m'):
                         compile(' '.join(uniq_warnings(p_cflags)), last_cflags_ts,
                             path + '/' + cfile + '.o', path + '/' + cfile + '.c')
                         gcc_list.append('%s/%s' % (path, cfile))
                     else:
-                        gcc_list.append(cfile)
+                        compile(' '.join(uniq_warnings(p_cflags)), last_cflags_ts,
+                            path + '/' + cfile + '.o', path + '/' + cfile)
+                        gcc_list.append('%s/%s' % (path, cfile))
 
                 libs += up.LIBS
 
@@ -453,7 +457,7 @@ class uConf(object):
             'core/setup_utils', 'core/clock', 'core/init', 'core/buffer',
             'core/plugins', 'core/lock', 'core/cache', 'core/daemons',
             'core/queue', 'core/event', 'core/signal', 'core/cluster',
-            'core/rpc', 'core/gateway', 'core/loop', 'lib/rbtree', 'core/rb_timers', 'core/uwsgi']
+            'core/rpc', 'core/gateway', 'core/loop', 'core/rb_timers', 'core/uwsgi']
         # add protocols
         self.gcc_list.append('proto/base')
         self.gcc_list.append('proto/uwsgi')
